/**
 * util.q.js
 * lulu.qiao on 2015-07-03 16:09:31.
 */
var Q = {};

/**
 * 替换path中的反斜线为正斜线
 * @param url
 * @returns {string}
 */

Q.urlSlash = function (url) {
    return (url ? url : '').replace(/\\/g, '/');
};

/**
 * 生成操作ID
 * @param len
 * @returns {string}
 */
Q.genOpId = function (len) {
    var a = (new Date()).getTime().toString(),
        l = a.length < len ? a.length : len;
    return a.substr(a.length - l);
};

/**
 * 分页弹动加载包裹器
 * @param f 回调
 * @param a 起始页码
 * @param b 每页条数
 * @returns {Function}
 */
Q.patchLoader = function (f, a, b) {
    var pg = a || 1;
    var sz = b || 10;
    return function () {
        f(pg++, sz);
    }
};

/**
 * 类真值
 * @param a
 * @returns {boolean}
 */
Q.isTrueLike = function (a) {
    if (typeof a == 'string') {
        console.warn('Bad argument(s) type.');
        return ['0','-0','null','false','NaN','undefined',''].indexOf(a) == -1;
    } else {
        return [0, -0, null, false, NaN, undefined].indexOf(a) == -1;
    }
};

/**
 * 格式化日期
 * @param {number} ts timeStamp
 * @param {string} sp separator
 * @returns {string}
 */
Q.formatDate = function (ts, sp) {
    if(typeof ts == "string"){
        ts = parseInt(ts);
    }
    ts = ts.length < 13 ? ts*1000 : ts;
    var dt = new Date(parseInt(ts)),
        res = [];
    res.push(dt.getFullYear(), dt.getMonth() + 1, dt.getDate());
    res.forEach(function (arrItem, index, arr) {
        arr[index] = arrItem.toString().length < 2 ? '0' + arrItem : arrItem;
    });
    return res.join(sp===undefined?".":sp);
};

/*=================== Common  ====================*/

Q.closeWindowByName = function (name,aniId) {
    aniId = aniId === undefined ? 0 : aniId;
    appcan.window.evaluateScript(name, "appcan.window.close(" + aniId + ")");
};

/*=================== Project ====================*/

/**
 * 是否已登录弹性福利模块
 * @returns {boolean}
 */
Q.isLoginShop = function () {
    return Q.isTrueLike(localStorage.getItem('isShop'));
};

/**
 * 卡登录
 * @returns {boolean}
 */
Q.isLoginCard = function () {
    return Q.isTrueLike(localStorage.getItem('isCard'));
};

/**
 * 凭证号登录
 * @returns {boolean}
 */
Q.isLoginVoucher = function () {
    return Q.isTrueLike(localStorage.getItem('isVoucher'));
};

/**
 * 处理请求返回的fieldErrors数据错误（非网络错误）
 * @param data
 * @param showAllErr
 * @param duration
 */
Q.showReqErr = function(data,showAllErr,duration){

    var errs=data.fieldErrors, res=[], keys=Object.keys||_.keys;
    if(!$.isPlainObject(errs) || !keys(errs).length){
        //TODO: errs is not an object OR has no own properties.
    }else{
        $.each(errs, function(i, o) {
            res.push(o.toString());
        });
    }
    var msg = showAllErr ? req.join('\r\n') : res[0];
    if(!msg){
        //TODO: msg is empty.
    }else{
        appcan.window.openToast(msg, duration||2000);
    }
};

/**
 * 以配置的前缀重试加载图片
 * @param {Element|EventTarget} elm
 * @param {Array} [path]
 */
Q.retrieveImg = function (elm, path) {
    //重试计数
    var retryCount = parseInt(elm.dataset.retry_count || 0);
    path = path ? path : window.IMG_RETRY_PREFIX || [];
    if (retryCount < path.length) {
        elm.src = path[retryCount] + elm.dataset.origin_src.replace(/^(\/)+/,'');
        elm.dataset.retry_count = parseInt(elm.dataset.retry_count || 0) + 1;
        //console.log('retry with: '+elm.src)
    }
};

/**
 * 图片加载错误事件处理
 * @param {Event} e
 */
Q.hImgError = function (e) {
    var elm = e.target,
        retryCount = parseInt(elm.dataset.retry_count || 0),
        regxUrl = (/(^http)|(^\/\/)/);//以[http]或[//]开头

    if (retryCount != 0) {
        //非第一次重试
    } else if (regxUrl.test(elm.dataset.origin_src)) {
        //绝对路径
        unbindErrorHandler(elm);
    }

    if (retryCount < window.IMG_RETRY_PREFIX.length) {
        Q.retrieveImg(elm);
    } else {
        unbindErrorHandler(elm);
    }

    //解除绑定
    function unbindErrorHandler(elm) {
        $(elm).off('error', Q.hImgError)
    }
};